//Known Issues
// onPlayPausedChanged and onAudioChanged fire double

/*******************
VARIABLES
*******************/
var creativeId = "VPAID Responsive"; //Use anything that uniquely identifies this creative
var creativeVersion = "2.0.0"; //0.0.1 during initial dev, 1.0.0 on release, x.x.x once released and receives updates
var lastModified = "2017-10-06";
var lastUploaded = "2017-10-06";
var debug = false;

var templateVersion = "2.0.27";

var ebVPAID, responsive, adId, rnd, uid;

var closeBtn, skipBtn, pausePlaybackBtn, resumePlaybackBtn, muteAudioBtn, unmuteAudioBtn, customInteractionBtn, videoClickOverlay, clickThroughBtn, logo;

/*******************
INITIALIZATION
*******************/

window.addEventListener("load", checkIfAdKitReady);

function checkIfAdKitReady(event) {
	adkit.onReady(initializeCreative);
}

function initializeCreative(event) {
	try {
		if (window.localPreview) {
			window.initializeLocalPreview(); //in localPreview.js
		}
	}
	catch (e) {}

	//Workaround (from QB6573) for Async EB Load where Modernizr isn't properly initialized
	typeof Modernizr == "object" && (Modernizr.touch = Modernizr.touch || "ontouchstart" in window);

	initializeGlobalVariables();
	window.registerInteraction = function () {}; //overwrite rI function because it will never actually be called
	addEventListeners();
	addVPAIDHandlers();

	//ebInitResponsive(["responsiveGroup"]);	//When not using script-based scaling, drop ebInitResponsive.
}

function initializeGlobalVariables() {
	ebVPAID = adkit.vpaid;

	adId = EB._adConfig.adId;
	rnd = EB._adConfig.rnd;
	uid = EB._adConfig.uid;
	
	closeBtn = document.getElementById("closeBtn");
	skipBtn = document.getElementById("skipBtn");
	
	muteAudioBtn = document.getElementById("muteAudioBtn");
	unmuteAudioBtn = document.getElementById("unmuteAudioBtn");
	pausePlaybackBtn = document.getElementById("pausePlaybackBtn");
	resumePlaybackBtn = document.getElementById("resumePlaybackBtn");
	
	customInteractionBtn = document.getElementById("customInteractionBtn");
	
	logo = document.getElementById("logo");
	videoClickOverlay = document.getElementById("videoClickOverlay");
	clickThroughBtn = document.getElementById("clickThroughBtn");
}

function addEventListeners() {
	closeBtn.addEventListener("click", onCloseAd);
	skipBtn.addEventListener("click", onSkipAd);

	pausePlaybackBtn.addEventListener("click", togglePlayback);
	resumePlaybackBtn.addEventListener("click", togglePlayback);
	muteAudioBtn.addEventListener("click", toggleAudio);
	unmuteAudioBtn.addEventListener("click", toggleAudio);

	customInteractionBtn.addEventListener("click", onCustomInteraction);

	logo.addEventListener("click", onClickThrough);
	videoClickOverlay.addEventListener("click", onClickThrough);
	clickThroughBtn.addEventListener("click", onClickThrough);
}

function registerInteraction() {
	//Register your automatic and user interactions here and the platform will parse them on workspace upload. This function will not be called.
	//Example: 'Left_Gutter_Viewed'
	//EB.automaticEventCounter("interactionName1");
	//EB.automaticEventCounter("interactionName2");

	//EB.userActionCounter("userActionName1");
	//EB.userActionCounter("userActionName2");
}

/*******************
EVENT HANDLERS
*******************/

function onClickThrough(event) {
	log('onClickThrough', event.target.id);
	ebVPAID.disableAutoClose();
	EB.clickthrough();
	ebVPAID.pause();
}

function onCloseAd() {
	log('onCloseAd');
	ebVPAID.closeAd();
}

function onSkipAd() {
	log('onSkipAd');
	ebVPAID.skipAd();
}

function onCustomInteraction() {
	log('onCustomInteraction');
	ebVPAID.disableAutoClose();
	EB.userActionCounter("CustomInteractionName");
}

/*******************
VPAID HANDLERS
*******************/
function addVPAIDHandlers() {
	ebVPAID.onVideoEvents(videoEventTriggered);
	ebVPAID.onCloseEnabled(closeEnabled);
	ebVPAID.onSkipEnabled(skipEnabled);
}

function videoEventTriggered(event) {
	switch (event.type) {
	case ebVPAID.VIDEO_EVENTS.ENDED:
	case ebVPAID.VIDEO_EVENTS.PAUSE:
	case ebVPAID.VIDEO_EVENTS.PLAY:
	case ebVPAID.VIDEO_EVENTS.PLAYING:
		onPlaybackChanged(event);
		break;
	case ebVPAID.VIDEO_EVENTS.VOLUME_CHANGE:
		onAudioChanged(event);
		break;
	}
}

function closeEnabled() {
	log('closeEnabled');
	closeBtn.style.display = "block";
	skipBtn.style.display = "none";
}

function skipEnabled() {
	log('skipEnabled');
	skipBtn.style.display = "block";
}


function toggleAudio() {
	ebVPAID.disableAutoClose();
	ebVPAID.muted = !ebVPAID.muted;
}

function togglePlayback(e) {
	ebVPAID.disableAutoClose();

	if (e.target == pausePlaybackBtn) {
		ebVPAID.pause();
	}
	else {
		ebVPAID.play();
	}
}

function onAudioChanged(event) {
	log('onAudioChanged', ebVPAID.muted ? "unmute" : "mute");
	if (ebVPAID.muted) {
		unmuteAudioBtn.parentNode.appendChild(unmuteAudioBtn);
	}
	else {
		muteAudioBtn.parentNode.appendChild(muteAudioBtn);
	}
}

function onPlaybackChanged(event) {
	log('onPlaybackChanged', event.type);
	if (event.type === ebVPAID.VIDEO_EVENTS.PLAY || event.type === ebVPAID.VIDEO_EVENTS.PLAYING) {
		pausePlaybackBtn.parentNode.appendChild(pausePlaybackBtn);
	}
	else {
		resumePlaybackBtn.parentNode.appendChild(resumePlaybackBtn);
	}
}

/*******************
UTILITIES
*******************/

function log() {
	if (debug) console.log.apply(console, arguments);
}

/*******************
Responsive Methods
*******************
If you're not comfortable using CSS to make your ad responsive, this code allows you to scale elements relative to their top/left position, while preserving their aspect ratio.
You can assign some elements in your ad (including groups within a div) to scale this way by adding their IDs to the responsiveElements array, and others not to scale by leaving them out.
*******************/

function ebInitResponsive(arrayOfResponsiveElementsByID) {
	responsive = new ebResponsive({
		containerId: "adContainer",
		originWidth: EB.getDimensions ? EB.getDimensions().width : 640,
		originHeight: EB.getDimensions ? EB.getDimensions().height : 360,
		responsiveElements: arrayOfResponsiveElementsByID || []
	});
}

function ebResponsive(obj) {
	log('ebResponsive', obj);
	this.originWidth = obj.originWidth;
	this.originHeight = obj.originHeight;
	this.container = document.getElementById(obj.containerId);
	this.container.style.width = obj.originWidth;
	this.container.style.height = obj.originHeight;
	this.responsiveElemData = [];
	var elem;
	for (var i = 0; i < obj.responsiveElements.length; i++) {
		elem = document.getElementById(obj.responsiveElements[i]);
		elem.style.transformOrigin = "top left";
		this.responsiveElemData[i] = elem;
	}

	window.addEventListener('resize', this.debounce(this.resize, 100).bind(this));
}

ebResponsive.prototype.debounce = function (func, wait, immediate) {
	var timeout;
	return function () {
		var context = this,
			args = arguments;
		var later = function () {
			timeout = null;
			if (!immediate)
				func.apply(context, args);
		};
		var callNow = immediate && !timeout;
		clearTimeout(timeout);
		timeout = setTimeout(later, wait);
		if (callNow)
			func.apply(context, args);
	};
};

ebResponsive.prototype.resize = function () {
	var windowWidth = window.innerWidth; //J--EB.getDimensions().width ?
	var windowHeight = window.innerHeight;
	var scaleX = windowWidth / this.originWidth;
	var scaleY = windowHeight / this.originHeight;
	var scale = (scaleX > scaleY) ? scaleY : scaleX;

	var currWidth = this.originWidth * scale;
	var currHeight = this.originHeight * scale;
	var top = (windowHeight - currHeight) / 2;
	var left = (windowWidth - currWidth) / 2;
	this.container.style.width = currWidth + "px";
	this.container.style.height = currHeight + "px";
	this.container.style.top = top + "px";
	this.container.style.left = left + "px";

	for (var i = 0; i < this.responsiveElemData.length; i++) {
		this.responsiveElemData[i].style.transform = "scale(" + scale + ")";
	}
};